/*
 * This file is part of Sponge, licensed under the MIT License (MIT).
 *
 * Copyright (c) SpongePowered <https://www.spongepowered.org>
 * Copyright (c) contributors
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package org.spongepowered.mod.mixin.core.entity;

import net.minecraft.entity.Entity;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.network.play.server.S07PacketRespawn;
import net.minecraft.network.play.server.S1DPacketEntityEffect;
import net.minecraft.potion.PotionEffect;
import net.minecraft.server.MinecraftServer;
import net.minecraft.world.WorldServer;
import net.minecraftforge.common.util.Constants;
import net.minecraftforge.fml.common.FMLCommonHandler;
import org.spongepowered.api.util.annotation.NonnullByDefault;
import org.spongepowered.api.world.Location;
import org.spongepowered.api.world.World;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Shadow;
import org.spongepowered.common.data.util.NbtDataUtil;
import org.spongepowered.common.interfaces.IMixinEntityPlayerMP;
import org.spongepowered.common.interfaces.entity.IMixinEntity;
import org.spongepowered.common.world.DimensionManager;


@NonnullByDefault
@Mixin(value = Entity.class, priority = 1001)
public abstract class MixinEntity implements IMixinEntity {

    // @formatter:off
    @Shadow public net.minecraft.world.World worldObj;

    @Shadow public abstract void setSize(float width, float height);
    @Shadow(remap = false)
    public abstract NBTTagCompound getEntityData();
    // @formatter:on

    @Override
    public final NBTTagCompound getSpongeData() {
        final NBTTagCompound data = this.getEntityData();
        if (!data.func_150297_b(NbtDataUtil.SPONGE_DATA, Constants.NBT.TAG_COMPOUND)) {
            data.func_74782_a(NbtDataUtil.SPONGE_DATA, new NBTTagCompound());
        }
        return data.func_74775_l(NbtDataUtil.SPONGE_DATA);
    }

    @SuppressWarnings("unchecked")
    public boolean teleportEntity(Entity entity, Location<World> location, int currentDim, int targetDim, boolean forced) {
        MinecraftServer mcServer = MinecraftServer.func_71276_C();
        final WorldServer fromWorld = mcServer.func_71218_a(currentDim);
        final WorldServer toWorld = mcServer.func_71218_a(targetDim);
        if (entity instanceof EntityPlayer) {
            fromWorld.func_73039_n().func_72787_a((EntityPlayerMP) entity);
            fromWorld.func_73040_p().func_72695_c((EntityPlayerMP) entity);
            mcServer.func_71203_ab().field_72404_b.remove(entity);
        } else {
            fromWorld.func_73039_n().func_72790_b(entity);
        }

        entity.field_70170_p.func_72973_f(entity);
        entity.field_71093_bK = targetDim;
        entity.func_70080_a(location.getX(), location.getY(), location.getZ(), 0, 0);
        if (forced) {
            while (!toWorld.func_72945_a(entity, entity.func_174813_aQ()).isEmpty() && entity.field_70163_u < 256.0D) {
                entity.func_70107_b(entity.field_70165_t, entity.field_70163_u + 1.0D, entity.field_70161_v);
            }
        }

        toWorld.field_73059_b.func_73158_c((int) entity.field_70165_t >> 4, (int) entity.field_70161_v >> 4);

        if (entity instanceof EntityPlayer) {
            EntityPlayerMP entityplayermp1 = (EntityPlayerMP) entity;

            // Support vanilla clients going into custom dimensions
            int clientDimension = DimensionManager.getClientDimensionToSend(toWorld.field_73011_w.func_177502_q(), toWorld, entityplayermp1);
            if (((IMixinEntityPlayerMP) entityplayermp1).usesCustomClient()) {
                DimensionManager.sendDimensionRegistration(toWorld, entityplayermp1, clientDimension);
            } else {
                // Send bogus dimension change for same worlds on Vanilla client
                if (currentDim != targetDim && (currentDim == clientDimension || targetDim == clientDimension)) {
                    entityplayermp1.field_71135_a.func_147359_a(
                            new S07PacketRespawn((byte) (clientDimension >= 0 ? -1 : 0), toWorld.func_175659_aa(), toWorld.func_72912_H().func_76067_t(),
                                    entityplayermp1.field_71134_c.func_73081_b()));
                }
            }

            entityplayermp1.field_71135_a.func_147359_a(
                    new S07PacketRespawn(clientDimension, toWorld.func_175659_aa(), toWorld.func_72912_H().func_76067_t(),
                            entityplayermp1.field_71134_c.func_73081_b()));
            entity.func_70029_a(toWorld);
            entity.field_70128_L = false;
            entityplayermp1.field_71135_a.func_147364_a(entityplayermp1.field_70165_t, entityplayermp1.field_70163_u, entityplayermp1.field_70161_v,
                    entityplayermp1.field_70177_z, entityplayermp1.field_70125_A);
            entityplayermp1.func_70095_a(false);
            mcServer.func_71203_ab().func_72354_b(entityplayermp1, toWorld);
            toWorld.func_73040_p().func_72683_a(entityplayermp1);
            toWorld.func_72838_d(entityplayermp1);
            mcServer.func_71203_ab().field_72404_b.add(entityplayermp1);
            entityplayermp1.field_71134_c.func_73080_a(toWorld);
            entityplayermp1.func_71116_b();
            entityplayermp1.func_70606_j(entityplayermp1.func_110143_aJ());
            for(Object effect : entityplayermp1.func_70651_bq()) {
                entityplayermp1.field_71135_a.func_147359_a(new S1DPacketEntityEffect(entityplayermp1.func_145782_y(), (PotionEffect) effect));
            }

            FMLCommonHandler.instance().firePlayerChangedDimensionEvent(entityplayermp1, currentDim, targetDim);
        } else {
            toWorld.func_72838_d(entity);
        }

        fromWorld.func_82742_i();
        toWorld.func_82742_i();
        return true;
    }

}
